#include <slow5/slow5.h>
#include <cassert>
#include <string>


// copied from squigulator/sim.c
#define ERROR(ignored, msg) fprintf(stderr, msg);
// set header attributes that are the same for the whole run
static void set_header_attributes(slow5_file_t *sp, int8_t rna, int8_t r10){

    slow5_hdr_t *header=sp->header;

    //add a header group attribute called run_id
    if (slow5_hdr_add("run_id", header) < 0){
        ERROR("%s","Error adding run_id attribute");
        exit(EXIT_FAILURE);
    }
    //add another header group attribute called asic_id
    if (slow5_hdr_add("asic_id", header) < 0){
        ERROR("%s","Error adding asic_id attribute");
        exit(EXIT_FAILURE);
    }
    //add another header group attribute called asic_id
    if (slow5_hdr_add("exp_start_time", header) < 0){
        ERROR("%s","Error adding asic_id attribute");
        exit(EXIT_FAILURE);
    }
    //add another header group attribute called flow_cell_id
    if (slow5_hdr_add("flow_cell_id", header) < 0){
        ERROR("%s","Error adding flow_cell_id attribute");
        exit(EXIT_FAILURE);
    }
    //add another header group attribute called experiment_type
    if (slow5_hdr_add("experiment_type", header) < 0){
        ERROR("%s","Error adding experiment_type attribute");
        exit(EXIT_FAILURE);
    }
    //add another header group attribute called sequencing_kit
    if (slow5_hdr_add("sequencing_kit", header) < 0){
        ERROR("%s","Error adding sequencing_kit attribute");
        exit(EXIT_FAILURE);
    }

    //set the run_id attribute to "run_0" for read group 0
    if (slow5_hdr_set("run_id", "run_0", 0, header) < 0){
        ERROR("%s","Error setting run_id attribute in read group 0");
        exit(EXIT_FAILURE);
    }
    //set the asic_id attribute to "asic_0" for read group 0
    if (slow5_hdr_set("asic_id", "asic_id_0", 0, header) < 0){
        ERROR("%s","Error setting asic_id attribute in read group 0");
        exit(EXIT_FAILURE);
    }
    //set the exp_start_time attribute to "2022-07-20T00:00:00Z" for read group 0
    if (slow5_hdr_set("exp_start_time", "2022-07-20T00:00:00Z", 0, header) < 0){
        ERROR("%s","Error setting exp_start_time attribute in read group 0");
        exit(EXIT_FAILURE);
    }
    //set the flow_cell_id attribute to "FAN00000" for read group 0
    if (slow5_hdr_set("flow_cell_id", "FAN00000", 0, header) < 0){
        ERROR("%s","Error setting flow_cell_id attribute in read group 0");
        exit(EXIT_FAILURE);
    }
    //set the experiment_type attribute to genomic_dna or rna for read group 0
    const char* experiment_type = rna ? "rna" : "genomic_dna" ;
    if (slow5_hdr_set("experiment_type", experiment_type, 0, header) < 0){
        ERROR("%s","Error setting experiment_type attribute in read group 0");
        exit(EXIT_FAILURE);
    }
    //sequencing kit
    const char* kit = rna ? "sqk-rna002" : (r10 ? "sqk-lsk114" : "sqk-lsk109") ;
    if (slow5_hdr_set("sequencing_kit", kit, 0, header) < 0){
        ERROR("%s","Error setting sequencing_kit attribute in read group 0");
        exit(EXIT_FAILURE);
    }
}
// define/add header fields that are specific to each record
static void set_header_aux_fields(slow5_file_t *sp){

    //add auxilliary field: channel number
    if (slow5_aux_add("channel_number", SLOW5_STRING, sp->header) < 0){
        ERROR("%s","Error adding channel_number auxilliary field\n");
        exit(EXIT_FAILURE);
    }

    //add auxilliary field: median_before
    if (slow5_aux_add("median_before", SLOW5_DOUBLE, sp->header) < 0) {
        ERROR("%s","Error adding median_before auxilliary field\n");
        exit(EXIT_FAILURE);
    }

    //add auxilliary field: read_number
    if(slow5_aux_add("read_number", SLOW5_INT32_T, sp->header) < 0){
        ERROR("%s","Error adding read_number auxilliary field\n");
        exit(EXIT_FAILURE);
    }
    //add auxilliary field: start_mux
    if(slow5_aux_add("start_mux", SLOW5_UINT8_T, sp->header) < 0){
        ERROR("%s","Error adding start_mux auxilliary field\n");
        exit(EXIT_FAILURE);
    }
    //add auxilliary field: start_time
    if(slow5_aux_add("start_time", SLOW5_UINT64_T, sp->header) < 0){
        ERROR("%s","Error adding start_time auxilliary field\n");
        exit(EXIT_FAILURE);
    }

}

void write_rudimentary_slow5_header(slow5_file_t* sp) {
    assert(sp != NULL);
    set_header_attributes(sp, false, false);
    set_header_aux_fields(sp);

    if (slow5_hdr_write(sp) < 0){
        fprintf(stderr, "Error writing header!\n");
        exit(EXIT_FAILURE);
    }
}

void create_file(std::string filename) {
    fprintf(stderr, "Creating file\n");
    
    slow5_file_t* sp = slow5_open(filename.c_str(), "w");
    assert(sp != NULL);
    write_rudimentary_slow5_header(sp);

    slow5_rec_t* rec = slow5_rec_init();
    assert(rec != NULL);

    // std::string read_id = "read1";
    // rec->read_id = strdup(read_id.c_str());
    rec->read_id = strdup("read1");
    rec->digitisation = 8192;
    rec->offset = 13.722261;
    rec->range = 1443.030273;
    rec->sampling_rate = 4000;
    rec->len_raw_signal = 3;
    rec->raw_signal = (int16_t*)(new int16_t[3]{1, 2, 3});

    auto current_file_ptr = sp;

    std::string channel_number = "1";
    uint32_t read_number = (uint32_t)(1);
    double median_before = 0.1;
    uint8_t start_mux = 1;
    uint64_t start_time = 100;
    if(slow5_aux_set_string(rec, "channel_number", channel_number.c_str(), current_file_ptr->header) < 0){
        fprintf(stderr,"Error setting channel_number auxiliary field\n");
        exit(EXIT_FAILURE);
    }
    if(slow5_aux_set(rec, "median_before", &median_before, current_file_ptr->header) < 0){
        fprintf(stderr,"Error setting median_before auxiliary field\n");
        exit(EXIT_FAILURE);
    }
    if(slow5_aux_set(rec, "read_number", &read_number, current_file_ptr->header) < 0){
        fprintf(stderr,"Error setting read_number auxiliary field\n");
        exit(EXIT_FAILURE);
    }
    if(slow5_aux_set(rec, "start_mux", &start_mux, current_file_ptr->header) < 0){
        fprintf(stderr,"Error setting start_mux auxiliary field\n");
        exit(EXIT_FAILURE);
    }
    if(slow5_aux_set(rec, "start_time", &start_time, current_file_ptr->header) < 0){
        fprintf(stderr,"Error setting start_time auxiliary field\n");
        exit(EXIT_FAILURE);
    }

    assert((slow5_write(rec, sp)));
    slow5_rec_free(rec);
    slow5_close(sp);

}
int main(int argc, char** argv) {
    create_file("test.blow5");
    create_file("test.slow5");
    return 0;
}